AutoBind.DefaultSettings = {
	BindOnOpenStore = false,
	BindOnCollect = false,
	MaxQuality = ITEM_QUALITY_MAX_VALUE,
}

local function Update_BindOnOpenStore(value)
	AutoBind.SavedVariables.BindOnOpenStore = value
	AutoBind.SetStoreEventActive(value)
end

local function Update_BindOnCollect(value)
	AutoBind.SavedVariables.BindOnCollect = value
	AutoBind.SetInventorySlotEventActive(value)
end

local function Update_MaxQuality(value)
	AutoBind.SavedVariables.MaxQuality = value
end

local function GetNewQualityChoices(colorize)
	local qualityChoices = {
		Choices = {},
		Values = {},
	}

	for i = ITEM_QUALITY_MAX_VALUE, ITEM_QUALITY_MAGIC, -1 do
		local qualityColor = GetItemQualityColor(i)
		local qualityName = GetString("SI_ITEMQUALITY", i)

		table.insert(qualityChoices.Choices, qualityColor:Colorize(qualityName))
		table.insert(qualityChoices.Values, i)
	end

	return qualityChoices
end

local function CreateSettingsCommands(savedVariables)
	local function ShowSettings()
		local settingsList = {}

		for k,v in pairs(AutoBind.DefaultSettings) do
			table.insert(settingsList, k .. ": " .. tostring(savedVariables[k]))
		end

		table.sort(settingsList, function(firstValue, secondValue) return firstValue < secondValue end)

		d("AutoBind Settings:")

		for i,v in ipairs(settingsList) do
			d("- " .. v)
		end
	end

	local validArguments = {
		["show"] = function() ShowSettings() end,
		["bindonopenstore true"] = function() Update_BindOnOpenStore(true) end,
		["bindonopenstore false"] = function() Update_BindOnOpenStore(false) end,
		["bindoncollect true"] = function() Update_BindOnCollect(true) end,
		["bindoncollect false"] = function() Update_BindOnCollect(false) end,
	}

	local infoOverride = {}
	local qualityChoices = GetNewQualityChoices()
	local maxQualityName = "maxquality "

	for i,v in ipairs(qualityChoices.Values) do
		local argName = maxQualityName .. string.lower(GetString("SI_ITEMQUALITY", v))

		validArguments[argName] = function() Update_MaxQuality(v) end
		infoOverride[argName] = maxQualityName .. string.lower(qualityChoices.Choices[i])
	end

	AutoBind.AddSlashCommand("/autobindsettings", validArguments, infoOverride)
end

local function CreateSettingsMenu(savedVariables)
	local optionsData = {}

	table.insert(optionsData, {
		type = "checkbox",
		name = "Bind On Open Store",
		tooltip = "If enabled, all unknown items in your inventory are automatically bound when a store window opens.",
		getFunc = function() return savedVariables.BindOnOpenStore end,
		setFunc = function(value) Update_BindOnOpenStore(value) end,
		default = AutoBind.DefaultSettings.BindOnOpenStore,
	} )

	table.insert(optionsData, {
		type = "checkbox",
		name = "Bind On Collect",
		tooltip = "If enabled, unknown items you collect are automatically bound.",
		getFunc = function() return savedVariables.BindOnCollect end,
		setFunc = function(value) Update_BindOnCollect(value) end,
		default = AutoBind.DefaultSettings.BindOnCollect,
	} )

	local qualityChoices = GetNewQualityChoices()

	table.insert(optionsData, {
		type = "dropdown",
		name = "Max Quality",
		tooltip = "Items with higher quality than selected are ignored.",
		choices = qualityChoices.Choices,
		choicesValues = qualityChoices.Values,
		getFunc = function() return savedVariables.MaxQuality end,
		setFunc = function(value) Update_MaxQuality(value) end,
		default = AutoBind.DefaultSettings.MaxQuality,
	} )

	local panelData = {
		type = "panel",
		name = AutoBind.AddonName,
		displayName = "Auto Bind",
		author = "ShinyBones",
		version = AutoBind.Version,
		slashCommand = "/autobindsettings",
		website = "https://www.esoui.com/downloads/info3217-AutoBind.html",
		registerForRefresh = false,
		registerForDefaults = true,
	}

	LibAddonMenu2:RegisterAddonPanel("AutoBindSettings", panelData)
	LibAddonMenu2:RegisterOptionControls("AutoBindSettings", optionsData)
end

function AutoBind.InitializeSettings()
	local savedVariables = AutoBind.SavedVariables

	AutoBind.SetStoreEventActive(savedVariables.BindOnOpenStore)
	AutoBind.SetInventorySlotEventActive(savedVariables.BindOnCollect)

	if LibAddonMenu2 then
		CreateSettingsMenu(savedVariables)
	else
		CreateSettingsCommands(savedVariables)
	end
end
